/* $Id: cpl_plot-test.c,v 1.13 2008/11/26 14:28:51 llundin Exp $
 *
 * This file is part of the ESO Common Pipeline Library
 * Copyright (C) 2001-2008 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: llundin $
 * $Date: 2008/11/26 14:28:51 $
 * $Revision: 1.13 $
 * $Name:  $
 */

/*-----------------------------------------------------------------------------
                                   Includes
 -----------------------------------------------------------------------------*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <float.h>
#include <math.h>

#include "cpl_plot.h"
#include "cpl_math_const.h"
#include "cpl_test.h"
#include "cpl_memory.h"
#include "cpl_vector.h"
#include "cpl_bivector.h"
#include "cpl_image.h"
#include "cpl_table.h"

/*-----------------------------------------------------------------------------
                                   Define
 -----------------------------------------------------------------------------*/

#define     VEC_SIZE    1024
#define     IMA_SIZE    100
#define     TAB_SIZE    100

/*-----------------------------------------------------------------------------
                                  Main
 -----------------------------------------------------------------------------*/
int main(void)
{
    cpl_vector      *   sinus ;
    cpl_vector      *   cosinus ;
    cpl_vector      *   x_deg ;
    cpl_vector      *   vec_array[3] ;
    cpl_bivector    *   sin_deg ;
    cpl_image       *   ima ;
    cpl_table       *   tab ;
    double          *   px ;
    double          *   py ;
    const char      *   names[] = {"COLUMN1", "COLUMN2", "COLUMN3"};
    int                 i ;

    cpl_test_init(PACKAGE_BUGREPORT, CPL_MSG_WARNING);

    /* Create the vector sinus */
    sinus = cpl_vector_new(VEC_SIZE) ;
    x_deg = cpl_vector_new(VEC_SIZE) ;

    px = cpl_vector_get_data(x_deg);
    py = cpl_vector_get_data(sinus);
    for (i=0 ; i<VEC_SIZE ; i++) {
        px[i] = i*CPL_MATH_2PI/VEC_SIZE * (360/CPL_MATH_2PI);
        py[i] = sin(i*CPL_MATH_2PI/VEC_SIZE);
    }

    /* Create the vector cosinus */
    cosinus = cpl_vector_new(VEC_SIZE) ;
    py = cpl_vector_get_data(cosinus);
    for (i=0 ; i<VEC_SIZE ; i++) py[i] = cos(i*CPL_MATH_2PI/VEC_SIZE);
    
    /* Create the vectors array */
    vec_array[0] = x_deg ;
    vec_array[1] = sinus ;
    vec_array[2] = cosinus ;

    /* Create the bivector */
    sin_deg = cpl_bivector_wrap_vectors(x_deg, sinus) ;

    /* Create the image */
    ima = cpl_image_fill_test_create(IMA_SIZE, IMA_SIZE) ; 

    /* Create the table */
    tab = cpl_table_new(TAB_SIZE) ;
    cpl_table_new_column(tab, names[0], CPL_TYPE_DOUBLE) ;
    cpl_table_new_column(tab, names[1], CPL_TYPE_DOUBLE) ;
    cpl_table_new_column(tab, names[2], CPL_TYPE_DOUBLE) ;
    for (i=0 ; i<TAB_SIZE ; i++) {
        cpl_table_set_double(tab, names[0], i, (double)(10*i+1)) ;
        cpl_table_set_double(tab, names[1], i, sin(10*i+1)/10*i+1) ;
        cpl_table_set_double(tab, names[2], i, log(i+1)/i+1) ;
    }

    /* No errors should have happened in the plot preparation */
    cpl_test_error(CPL_ERROR_NONE);
 
    /* No plotting is done unless the message level is modified */
    /* cpl_msg_set_level(CPL_MSG_DEBUG) ; */
    if (cpl_msg_get_level() <= CPL_MSG_INFO) {

        /* An error may occur, e.g. in the absence of gnuplot.
           The unit test should still succeed, and it is only verified
           that the return code equals the CPL error code. */
        cpl_error_code error;

        /* PLOT sinus */
          error = cpl_plot_vector("", "w lines", "", sinus) ;
          cpl_test_error(error);
    
        /* PLOT sinus in degrees */
          error = cpl_plot_bivector("", "w lines", "", sin_deg) ;
          cpl_test_error(error);

        /* PLOT sinus and cosinus */
          error = cpl_plot_vectors("", "w lines", "",
                                   (const cpl_vector**)vec_array, 3);
          cpl_test_error(error);

        /* PLOT the image */
          error = cpl_plot_image("", "w lines", "", ima) ;
          cpl_test_error(error);

        /* PLOT the table */
          error = cpl_plot_column("set grid;", "w lines", "", tab, names[0],
                  names[1]);
          error = cpl_plot_column("set grid;", "w lines", "", tab, NULL,
                  names[2]);
          error = cpl_plot_columns("set grid;", "w lines", "", tab, names, 3);
          cpl_test_error(error);

        /* Example of some plotting using the options */
          error = cpl_plot_image_row("set xlabel 'X Position (pixels)';"
                                     "set ylabel 'Counts (ADU)';",  "", "", 
                                     ima, 50, 52, 1) ;
          cpl_test_error(error);
    }

    /* Free and return */
    cpl_table_delete(tab) ;
    cpl_image_delete(ima) ;
    cpl_bivector_unwrap_vectors(sin_deg) ;
    cpl_vector_delete(sinus) ;
    cpl_vector_delete(cosinus) ;
    cpl_vector_delete(x_deg) ;

    return cpl_test_end(0);
}

/**@}*/
